/*
 * Copyright (c) 2010-2011 by inovat, innovative systeme - verkehr - tunnel - technik, Dipl.-Ing. H. C. Kniss
 *
 * This file is part of de.inovat.dua.pufferlzzs.VewDeLve255
 *
 * de.inovat.dua.pufferlzzs.VewDeLve255 is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * de.inovat.dua.pufferlzzs.VewDeLve255 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * <p>
 * You should have received a copy of the GNU General Public License
 * along with de.inovat.dua.pufferlzzs.VewDeLve255; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Contact Information:
 * inovat, Dipl.-Ing. H. C. Kniss
 * Koelner Strasse 30
 * D-50859 Koeln
 * +49 (0)2234 4301 800
 * info@invat.de
 * www.inovat.de
 */



package de.inovat.dua.pufferlzzs;

//~ NICHT JDK IMPORTE =========================================================

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientSenderInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.OneSubscriptionPerSendData;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.SendSubscriptionNotConfirmed;
import de.bsvrz.dav.daf.main.SenderRole;
import de.bsvrz.dav.daf.main.archive.ArchiveRequestManager;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

import java.text.SimpleDateFormat;
import java.util.Set;

//~ KLASSEN ===================================================================

/**
 * Verwaltungsklasse je DeLve mit DeNummer = 255, welches für genau ein Objekt den Pufferabruf für einen Auftrag
 * ausführt.
 * <p>
 * Das Verwaltungsobjekt fragt sowohl die Archivdaten ab, berechnet die ggf. nachzufordernden Langzeitdaten und sendet
 * die Pufferabfrage.
 * <p>
 * Weiterhin gibt die Verwaltungsklasse eine aussagekräftige Meldung zurück, die den vollständigen Ablauf der
 * Aggregation beschreibt.
 *
 * @author inovat, innovative systeme - verkehr - tunnel - technik
 * @author Dipl.-Ing. Hans Christian Kniß (HCK)
 * @version $Revision$ / $Date$ / ($Author$)
 */
public class VewDeLve255 implements ClientSenderInterface {

    /** DebugLogger für Debug-Ausgaben. */
    private static final Debug debug = Debug.getLogger();

    /** Datumsformattierer  zur Ausgabe von Datum und Uhrzeit. */
    private static SimpleDateFormat datumsFormatierer = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss,SSS");

    /** Archivsystem */
    private static ArchiveRequestManager _archiv;
    private static Aspect                _aspExterneErfassung;
    private static Aspect                _aspTlsAbruf;
    private static AttributeGroup        _atgFsDTV;
    private static AttributeGroup        _atgPufferAbrufLve;

    /** Datenverteilerverbindung */
    private static ClientDavInterface _dav;

    /** Die Konfiguration des DaV. */
    private static DataModel _konfiguration;

    //~ FELDER ================================================================

    /** Das Systemobjekt, für welches diese Verwaltungsklasse die Daten berechnet */
    SystemObject _deLve255;

    /** Intervalldauer in Stunden, mit denen die Daten am FS erfasst wurden. */
    private int _intervallDauerInStunden;

    /** Menge mit den FS, die dem LVE DE 255 zugeordnet sind (für die die Pufferabfrage durchgeführt werden muss). */
    Set<SystemObject> _mengeFs;
    private int       _nachberechnungInStunden;

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    /**
     * Konstruktor der Verwaltungs Klasse.
     *
     * @param dav                     Die Datenverteilerverbindung
     * @param deLve255                Das Systemobjekt, für welches diese Verwaltungsklasse die Daten berechnen soll
     * @param mengeFs                 Menge FS
     * @param intervallDauerInStunden Intervalldauer in Stunden für die erfassten FS-Daten
	 * @param nachberechnungInStunden Nachberechnung in Stunden
	 */
    public VewDeLve255(ClientDavInterface dav, SystemObject deLve255, Set<SystemObject> mengeFs, int intervallDauerInStunden, int nachberechnungInStunden) {
        _dav                     = dav;
        _deLve255                = deLve255;
        _mengeFs                 = mengeFs;
        _intervallDauerInStunden = intervallDauerInStunden;
        _nachberechnungInStunden = nachberechnungInStunden;
        _konfiguration           = _dav.getDataModel();
        _archiv                  = _dav.getArchive();
        _atgFsDTV                = _konfiguration.getAttributeGroup("atg.verkehrsDatenLangZeitIntervallGeschwindigkeitVerteilungFs");
        _aspExterneErfassung     = _konfiguration.getAspect("asp.externeErfassung");
        _atgPufferAbrufLve       = _konfiguration.getAttributeGroup("atg.tlsLveAbrufPufferInhalt");
        _aspTlsAbruf             = _konfiguration.getAspect("asp.tlsAbruf");

        // Sendeanmeldung für Zielattributgruppe (Pufferabfrage LVE der TLS)
        try {
            _dav.subscribeSender(this, _deLve255, new DataDescription(_atgPufferAbrufLve, _aspTlsAbruf), SenderRole.sender());
        }
        catch (OneSubscriptionPerSendData e) {
            debug.warning(String.format("Doppelanmeldung für [%s] (anderer Sender vorhanden?). Pufferabruf wird für Objekt nicht weiter berücksichtigt.", _deLve255.getPid()), e);
        }
    }

    //~ METHODEN ==============================================================

    /**
     * Methode, die den eigentlichen Auftrag ausführt.
     *
     * @param auftrag Der auszuführende Auftrag.
     *
     * @return Meldung, die den Ablauf der Ausführung des Auftrags enthält. Wird für die Statusdarstellung verwendet.
     */
    public String ausfuehren(Auftrag auftrag) {
        DataDescription datenBeschreibung = new DataDescription(_atgPufferAbrufLve, _aspTlsAbruf);
        Data            datenPufferAbruf  = _dav.createData(_atgPufferAbrufLve);

        // ToDo: 1. Prüfen, ob im Nachberechnungszeitraum für alle FS die Daten vorliegen, ggf. Archivabfrage
        // ToDo: 2. Ggf. Pufferabruf für die Lücken durchführen
        // 3. Standardpufferanfrage für das aktuelle Intervall abfragen
        datenPufferAbruf.getTimeValue("Zeitpunkt").setMillis(auftrag.getIntervallBeginn());
        datenPufferAbruf.getUnscaledValue("Anzahl").set(1);

        try {
            _dav.sendData(new ResultData(_deLve255, datenBeschreibung, System.currentTimeMillis(), datenPufferAbruf, false));
        }
        catch (SendSubscriptionNotConfirmed e) {
            debug.warning(String.format("Sendeversuch für Pufferabfrage wegen fehlender Sendeanmeldung fehlgeschlagen: [%s]", datenBeschreibung), e);
        }

        return auftrag.toString();
    }

    /**
     * Sendesteuerung des Datenverteilers an die Applikation. Diese Methode muss von der Applikation implementiert
     * werden, um den Versand von Daten zu starten bzw. anzuhalten. Der Datenverteiler signalisiert damit einer Quelle
     * oder einem Sender dass mindestens ein Abnehmer bzw. kein Abnehmer mehr für die zuvor angemeldeten Daten vorhanden
     * ist. Die Quelle wird damit aufgefordert den Versand von Daten zu starten bzw. zu stoppen.
     *
     * @param object          Das in der zugehörigen Sendeanmeldung angegebene Objekt, auf das sich die Sendesteuerung
     *                        bezieht.
     * @param dataDescription Beschreibende Informationen zu den angemeldeten Daten auf die sich die Sendesteuerung
     *                        bezieht.
     * @param state           Status der Sendesteuerung. Kann einen der Werte <code>START_SENDING</code>,
     *                        <code>STOP_SENDING</code>, <code>STOP_SENDING_NO_RIGHTS</code>,
     *                        <code>STOP_SENDING_NOT_A_VALID_SUBSCRIPTION</code> enthalten.
     *
     * @see #START_SENDING
     * @see #STOP_SENDING
     * @see #STOP_SENDING_NO_RIGHTS
     * @see #STOP_SENDING_NOT_A_VALID_SUBSCRIPTION
     */
    @Override
    public void dataRequest(SystemObject object, DataDescription dataDescription, byte state) {

        // Wird nicht unterstützt. Ergebnisdaten werden immer gesendet.
    }

    //~ GET METHODEN ==========================================================

    /**
     * Liefert die Pid des dieser Verwaltungseinheit zugeordnete LZZS Systemobjekt.
     *
     * @return Pid des dieser Verwaltungseinheit zugeordnete LZZS Systemobjekt.
     */
    public String getPid() {
        return _deLve255.getPid();
    }

    /**
     * Diese Methode muss von der Applikation implementiert werden, um zu signalisieren, ob Sendesteuerungen erwünscht
     * sind und mit der Methode <code>dataRequest</code> verarbeitet werden. In der Implementierung dieser Methode
     * dürfen keine synchronen Aufrufe, die auf Telegramme vom Datenverteiler warten (wie z.B. Konfigurationsanfragen)
     * durchgeführt werden, da ansonsten ein Deadlock entsteht.
     *
     * @param object          Das in der zugehörigen Sendeanmeldung angegebene System-Objekt.
     * @param dataDescription Die in der zugehörigen Sendeanmeldung angegebenen beschreibenden Informationen der
     *                        angemeldeten Daten.
     *
     * @return <code>true</code>, falls Sendesteuerungen gewünscht sind, sonst <code>false</code>.
     *
     * @see #dataRequest
     */
    @Override
    public boolean isRequestSupported(SystemObject object, DataDescription dataDescription) {
        return false;
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 08.12.10
